@@ -66,6 +66,7 @@ def get_adapter_info(adapter):
     pipeline.get_state()
     bus = pipeline.get_bus()
     info = {}
+    success = False
     while bus.have_pending():
         msg = bus.pop()
         if msg.type == gst.MESSAGE_ELEMENT and msg.src == dvbelement:
@@ -73,9 +74,13 @@ def get_adapter_info(adapter):
             if structure.get_name() == "dvb-adapter":
                 info["type"] = structure["type"]
                 info["name"] = structure["name"]
+                success = True
                 break
+        elif msg.type == gst.MESSAGE_ERROR:
+            info = msg.structure["debug"]
+            global_error_handler(info)
     pipeline.set_state(gst.STATE_NULL)
-    return info
+    return (success, info)
 
 def get_dvb_devices(reply_handler, error_handler):
     def find_devices_handler(objects):
@@ -122,26 +127,51 @@ class DVBManagerClient(gobject.GObject):
         self.manager.connect_to_signal("GroupRemoved", self.on_group_removed)
         
     def get_scanner_for_device(self, adapter, frontend):
-        objpath, scanner_iface = self.manager.GetScannerForDevice (adapter, frontend)
-        return DVBScannerClient(objpath, scanner_iface)
+        objpath, scanner_iface, success = self.manager.GetScannerForDevice (adapter, frontend)
+        if success:
+            return DVBScannerClient(objpath, scanner_iface)
+        else:
+            return None
         
     def get_device_group(self, group_id):
-        path = self.manager.GetDeviceGroup(group_id)
-        return DVBDeviceGroupClient(path)
+        path, success = self.manager.GetDeviceGroup(group_id)
+        if success:
+            return DVBDeviceGroupClient(path)
+        else:
+            return None
         
-    def get_registered_device_groups(self, reply_handler, error_handler):
+    def get_registered_device_groups(self, **kwargs):
+        if "reply_handler" in kwargs:
+            reply_handler = kwargs["reply_handler"]
+        else:
+            reply_handler = None
+
         def groups_handler(paths):
             reply_handler([DVBDeviceGroupClient(path) for path in paths])
-        self.manager.GetRegisteredDeviceGroups(reply_handler=groups_handler, error_handler=error_handler)
+        
+        if reply_handler != None:
+            self.manager.GetRegisteredDeviceGroups(reply_handler=groups_handler,
+                error_handler=kwargs["error_handler"])
+        else:
+            return [DVBDeviceGroupClient(path) for path in self.manager.GetRegisteredDeviceGroups()]
        
     def add_device_to_new_group (self, adapter, frontend, channels_file, recordings_dir, name, **kwargs):
         return self.manager.AddDeviceToNewGroup(adapter, frontend, channels_file, recordings_dir, name, **kwargs)
        
-    def get_name_of_registered_device(self, adapter, frontend):
-        return self.manager.GetNameOfRegisteredDevice(adapter, frontend)
+    def get_name_of_registered_device(self, adapter, frontend, **kwargs):
+        return self.manager.GetNameOfRegisteredDevice(adapter, frontend, **kwargs)
         
-    def get_device_group_size(self):
-        return self.manager.GetDeviceGroupSize()
+    def get_device_group_size(self, **kwargs):
+        return self.manager.GetDeviceGroupSize(**kwargs)
+        
+    def get_channel_groups(self, **kwargs):
+        return self.manager.GetChannelGroups(**kwargs)
+        
+    def add_channel_group(self, name, **kwargs):
+        return self.manager.AddChannelGroup(name, **kwargs)
+        
+    def remove_channel_group(self, group_id, **kwargs):
+        return self.manager.RemoveChannelGroup(group_id, **kwargs)
     
     def on_group_added(self, group_id):
         self.emit("group-added", group_id)
@@ -181,34 +211,37 @@ class DVBDeviceGroupClient(gobject.GObject):
     def add_device (self, adapter, frontend, **kwargs):
         return self.devgroup.AddDevice(adapter, frontend, **kwargs)
         
-    def remove_device(self, adapter, frontend):
-        return self.devgroup.RemoveDevice(adapter, frontend)
+    def remove_device(self, adapter, frontend, **kwargs):
+        return self.devgroup.RemoveDevice(adapter, frontend, **kwargs)
     
     def get_channel_list(self):
         path = self.devgroup.GetChannelList()
         return DVBChannelListClient(path)
     
-    def get_members(self):
-        return self.devgroup.GetMembers()
+    def get_members(self, **kwargs):
+        return self.devgroup.GetMembers(**kwargs)
         
-    def get_name(self):
-        return self.devgroup.GetName()
+    def get_name(self, **kwargs):
+        return self.devgroup.GetName(**kwargs)
     
-    def set_name(self, name):
-        return self.devgroup.SetName(name)
+    def set_name(self, name, **kwargs):
+        return self.devgroup.SetName(name, **kwargs)
         
-    def get_type(self):
-        return self.devgroup.GetType()
+    def get_type(self, **kwargs):
+        return self.devgroup.GetType(**kwargs)
         
     def get_schedule(self, channel_sid):
-        path = self.devgroup.GetSchedule(channel_sid)
-        return DVBScheduleClient(path)
+        path, success = self.devgroup.GetSchedule(channel_sid)
+        if success:
+            return DVBScheduleClient(path)
+        else:
+            return None
         
-    def get_recordings_directory (self):
-        return self.devgroup.GetRecordingsDirectory()
+    def get_recordings_directory (self, **kwargs):
+        return self.devgroup.GetRecordingsDirectory(**kwargs)
         
-    def set_recordings_directory (self, location):
-        return self.devgroup.SetRecordingsDirectory(location)
+    def set_recordings_directory (self, location, **kwargs):
+        return self.devgroup.SetRecordingsDirectory(location, **kwargs)
      
     def on_device_added(self, adapter, frontend):
         self.emit("device-added", adapter, frontend)
@@ -236,17 +269,17 @@ class DVBScannerClient(gobject.GObject):
         self.scanner.connect_to_signal ("ChannelAdded", self.on_channel_added)
         self.scanner.connect_to_signal ("Destroyed", self.on_destroyed)
         
-    def add_scanning_data(self, data):
-        self.scanner.AddScanningData (*data)
+    def add_scanning_data(self, data, **kwargs):
+        self.scanner.AddScanningData (*data, **kwargs)
         
     def add_scanning_data_from_file(self, path, **kwargs):
         return self.scanner.AddScanningDataFromFile(path, **kwargs)
         
-    def run(self):
-        self.scanner.Run()
+    def run(self, **kwargs):
+        self.scanner.Run(**kwargs)
         
-    def destroy(self):
-        self.scanner.Destroy()
+    def destroy(self, **kwargs):
+        self.scanner.Destroy(**kwargs)
         
     def write_channels_to_file(self, channel_sids, channelfile, **kwargs):
         self.scanner.WriteChannelsToFile(channel_sids, channelfile, **kwargs)
@@ -285,32 +318,32 @@ class DVBRecordingsStoreClient(gobject.GObject):
     def get_recordings(self, **kwargs):
         return self.recstore.GetRecordings(**kwargs)
         
-    def get_location(self, rid):
-        return self.recstore.GetLocation(rid)
+    def get_location(self, rid, **kwargs):
+        return self.recstore.GetLocation(rid, **kwargs)
         
-    def get_name(self, rid):
-        return self.recstore.GetName(rid)
+    def get_name(self, rid, **kwargs):
+        return self.recstore.GetName(rid, **kwargs)
         
-    def get_description(self, rid):
-        return self.recstore.GetDescription(rid)
+    def get_description(self, rid, **kwargs):
+        return self.recstore.GetDescription(rid, **kwargs)
         
-    def get_length(self, rid):
-        return self.recstore.GetLength(rid)
+    def get_length(self, rid, **kwargs):
+        return self.recstore.GetLength(rid, **kwargs)
         
-    def get_start_time(self, rid):
-        return self.recstore.GetStartTime(rid)
+    def get_start_time(self, rid, **kwargs):
+        return self.recstore.GetStartTime(rid, **kwargs)
         
-    def get_start_timestamp(self, rid):
-        return self.recstore.GetStartTimestamp(rid)
+    def get_start_timestamp(self, rid, **kwargs):
+        return self.recstore.GetStartTimestamp(rid, **kwargs)
         
     def delete(self, rid, **kwargs):
         return self.recstore.Delete(rid, **kwargs)
         
-    def get_channel_name(self, rid):
-        return self.recstore.GetChannelName(rid)
+    def get_channel_name(self, rid, **kwargs):
+        return self.recstore.GetChannelName(rid, **kwargs)
         
-    def get_all_informations(self, rid):
-        return self.recstore.GetAllInformations(rid)
+    def get_all_informations(self, rid, **kwargs):
+        return self.recstore.GetAllInformations(rid, **kwargs)
         
     def on_changed(self, rid, typeid):
         self.emit("changed", rid, typeid)
@@ -339,50 +372,56 @@ class DVBRecorderClient(gobject.GObject):
     def get_path(self):
         return self.object_path
         
-    def add_timer (self, channel, year, month, day, hour, minute, duration):
-        return self.recorder.AddTimer(channel, year, month, day, hour, minute, duration)
+    def add_timer (self, channel, year, month, day, hour, minute, duration, **kwargs):
+        return self.recorder.AddTimer(channel, year, month, day, hour, minute, duration, **kwargs)
         
-    def add_timer_with_margin (self, channel, year, month, day, hour, minute, duration):
-        return self.recorder.AddTimerWithMargin(channel, year, month, day, hour, minute, duration)
+    def add_timer_with_margin (self, channel, year, month, day, hour, minute, duration, **kwargs):
+        return self.recorder.AddTimerWithMargin(channel, year, month, day, hour, minute, duration, **kwargs)
         
-    def add_timer_for_epg_event(self, event_id, channel_sid):
-        return self.recorder.AddTimerForEPGEvent(event_id, channel_sid)
+    def add_timer_for_epg_event(self, event_id, channel_sid, **kwargs):
+        return self.recorder.AddTimerForEPGEvent(event_id, channel_sid, **kwargs)
         
-    def delete_timer(self, tid):
-        return self.recorder.DeleteTimer(tid)
+    def delete_timer(self, tid, **kwargs):
+        return self.recorder.DeleteTimer(tid, **kwargs)
         
     def get_timers(self, **kwargs):
         return self.recorder.GetTimers(**kwargs)
         
-    def get_start_time(self, tid):
-        return self.recorder.GetStartTime(tid)
-        
-    def get_end_time(self, tid):
-        return self.recorder.GetEndTime(tid)
-        
-    def get_duration(self, tid):
-        return self.recorder.GetDuration(tid)
-        
-    def get_channel_name(self, tid):
-        return self.recorder.GetChannelName(tid)
+    def get_start_time(self, tid, **kwargs):
+        return self.recorder.GetStartTime(tid, **kwargs)
 
-    def get_title(self, tid):
-        return self.recorder.GetTitle(tid)
+    def set_start_time(self, tid, year, month, day, hour, minute, **kwargs):
+        return self.recorder.SetStartTime (tid, year, month, day, hour, minute)
+        
+    def get_end_time(self, tid, **kwargs):
+        return self.recorder.GetEndTime(tid, **kwargs)
+        
+    def get_duration(self, tid, **kwargs):
+        return self.recorder.GetDuration(tid, **kwargs)
 
-    def get_all_informations(self, tid):
-        return self.recorder.GetAllInformations(tid)
+    def set_duration(self, tid, duration, **kwargs):
+        return self.recorder.SetDuration(tid, duration, **kwargs)
         
-    def get_active_timers(self):
-        return self.recorder.GetActiveTimers()
+    def get_channel_name(self, tid, **kwargs):
+        return self.recorder.GetChannelName(tid, **kwargs)
+
+    def get_title(self, tid, **kwargs):
+        return self.recorder.GetTitle(tid, **kwargs)
+
+    def get_all_informations(self, tid, **kwargs):
+        return self.recorder.GetAllInformations(tid, **kwargs)
         
-    def is_timer_active(self, tid):
-        return self.recorder.IsTimerActive(tid)
+    def get_active_timers(self, **kwargs):
+        return self.recorder.GetActiveTimers(**kwargs)
         
-    def has_timer(self, year, month, day, hour, minute, duration):
-        return self.recorder.HasTimer(year, month, day, hour, minute, duration)
+    def is_timer_active(self, tid, **kwargs):
+        return self.recorder.IsTimerActive(tid, **kwargs)
         
-    def has_timer_for_event(self, event_id, channel_sid):
-        return self.recorder.HasTimerForEvent(event_id, channel_sid)
+    def has_timer(self, year, month, day, hour, minute, duration, **kwargs):
+        return self.recorder.HasTimer(year, month, day, hour, minute, duration, **kwargs)
+        
+    def has_timer_for_event(self, event_id, channel_sid, **kwargs):
+        return self.recorder.HasTimerForEvent(event_id, channel_sid, **kwargs)
         
     def on_recording_started(self, timer_id):
         self.emit("recording-started", timer_id)
@@ -415,21 +454,30 @@ class DVBChannelListClient:
     def get_tv_channels(self, **kwargs):
         return self.channels.GetTVChannels(**kwargs)
         
-    def get_channel_name(self, cid):
-        return self.channels.GetChannelName(cid)
+    def get_channel_name(self, cid, **kwargs):
+        return self.channels.GetChannelName(cid, **kwargs)
         
-    def get_channel_network(self, cid):
-        return self.channels.GetChannelNetwork(cid)
+    def get_channel_network(self, cid, **kwargs):
+        return self.channels.GetChannelNetwork(cid, **kwargs)
         
-    def is_radio_channel(self, cid):
-        return self.channels.IsRadioChannel(cid)
+    def is_radio_channel(self, cid, **kwargs):
+        return self.channels.IsRadioChannel(cid, **kwargs)
         
-    def get_channel_url(self, cid):
-        return self.channels.GetChannelURL(cid)
+    def get_channel_url(self, cid, **kwargs):
+        return self.channels.GetChannelURL(cid, **kwargs)
         
     def get_channel_infos(self, **kwargs):
         return self.channels.GetChannelInfos(**kwargs)
         
+    def get_channels_of_group(self, group_id, **kwargs):
+        return self.channels.GetChannelsOfGroup(group_id, **kwargs)
+        
+    def add_channel_to_group(self, cid, group_id, **kwargs):
+        return self.channels.AddChannelToGroup(cid, group_id, **kwargs)
+        
+    def remove_channel_from_group(self, cid, group_id, **kwargs):
+        return self.channels.RemoveChannelFromGroup(cid, group_id, **kwargs)
+        
 class DVBScheduleClient(gobject.GObject):
 
     def __init__(self, object_path):
@@ -462,35 +510,35 @@ class DVBScheduleClient(gobject.GObject):
     def get_informations(self, eid, **kwargs):
         return self.schedule.GetInformations(eid, **kwargs)
         
-    def now_playing(self):
-        return self.schedule.NowPlaying()
+    def now_playing(self, **kwargs):
+        return self.schedule.NowPlaying(**kwargs)
         
-    def next(self, eid):
-        return self.schedule.Next(eid)
+    def next(self, eid, **kwargs):
+        return self.schedule.Next(eid, **kwargs)
         
-    def get_name(self, eid):
-        return self.schedule.GetName(eid)
+    def get_name(self, eid, **kwargs):
+        return self.schedule.GetName(eid, **kwargs)
         
-    def get_short_description(self, eid):
-        return self.schedule.GetShortDescription(eid)
+    def get_short_description(self, eid, **kwargs):
+        return self.schedule.GetShortDescription(eid, **kwargs)
         
-    def get_extended_description(self, eid):
-        return self.schedule.GetExtendedDescription(eid)
+    def get_extended_description(self, eid, **kwargs):
+        return self.schedule.GetExtendedDescription(eid, **kwargs)
         
-    def get_duration(self, eid):
-        return self.schedule.GetDuration(eid)
+    def get_duration(self, eid, **kwargs):
+        return self.schedule.GetDuration(eid, **kwargs)
         
     def get_local_start_time(self, eid):
         return self.schedule.GetLocalStartTime(eid)
         
-    def get_local_start_timestamp(self, eid):
-        return self.schedule.GetLocalStartTimestamp(eid)
+    def get_local_start_timestamp(self, eid, **kwargs):
+        return self.schedule.GetLocalStartTimestamp(eid, **kwargs)
         
-    def is_running(self, eid):
-        return self.schedule.IsRunning(eid)
+    def is_running(self, eid, **kwargs):
+        return self.schedule.IsRunning(eid, **kwargs)
         
-    def is_scrambled(self, eid):
-        return self.schedule.IsScrambled(eid)
+    def is_scrambled(self, eid, **kwargs):
+        return self.schedule.IsScrambled(eid, **kwargs)
         
 if __name__ == '__main__':
     loop = gobject.MainLoop()
@@ -557,10 +605,10 @@ if __name__ == '__main__':
             print "Location", recstore.get_location(rid)
             print "Start", recstore.get_start_time(rid)
             print recstore.get_start_timestamp(rid)
-            print "Length", recstore.get_length(rid)    
+            print "Length", recstore.get_length(rid)
             print "Name", recstore.get_name (rid)
             print "Desc", recstore.get_description(rid)
     
-    dev_groups = manager.get_registered_device_groups(reply_handler=device_handler)
+    dev_groups = manager.get_registered_device_groups(reply_handler=device_handler, error_handler=global_error_handler)
     
     loop.run()
