@@ -17,7 +17,6 @@
 # along with GNOME DVB Daemon.  If not, see <http://www.gnu.org/licenses/>.
 
 import gtk
-import pango
 import gobject
 from gettext import gettext as _
 import gnomedvb
@@ -25,32 +24,15 @@ from gnomedvb.ui.widgets.ChannelsStore import ChannelsStore
 from gnomedvb.ui.widgets.ChannelsView import ChannelsView
 from gnomedvb.ui.widgets.RunningNextStore import RunningNextStore
 from gnomedvb.ui.widgets.RunningNextView import RunningNextView
+from gnomedvb.ui.widgets.SchedulePaned import SchedulePaned
 from gnomedvb.ui.widgets.ScheduleStore import ScheduleStore
-from gnomedvb.ui.widgets.ScheduleView import ScheduleView
+from gnomedvb.ui.widgets.HelpBox import HelpBox
+from gnomedvb.ui.channellisteditor.ChannelListEditorDialog import ChannelListEditorDialog
 from gnomedvb.ui.timers.EditTimersDialog import EditTimersDialog
-from gnomedvb.ui.timers.TimerDialog import NoTimerCreatedDialog
+from gnomedvb.ui.timers.MessageDialogs import TimerFailureDialog
 from gnomedvb.ui.preferences.Preferences import Preferences
 from gnomedvb.ui.recordings.RecordingsDialog import RecordingsDialog
-
-class HelpBox(gtk.EventBox):
-
-    def __init__(self):
-        gtk.EventBox.__init__(self)
-        self.modify_bg(gtk.STATE_NORMAL, self.style.base[gtk.STATE_NORMAL])
-                
-        frame = gtk.Frame()
-        frame.set_shadow_type(gtk.SHADOW_IN)
-        self.add(frame)
-        
-        self._helpview = gtk.Label()
-        self._helpview.set_ellipsize(pango.ELLIPSIZE_END)
-        self._helpview.set_alignment(0.50, 0.50)
-        frame.add(self._helpview)
-        
-    def set_markup(self, helptext):
-        self._helpview.set_markup("<span foreground='grey50'>%s</span>" % helptext)
-
-
+   
 class ControlCenterWindow(gtk.Window):
 
     def __init__(self, model):
@@ -59,6 +41,7 @@ class ControlCenterWindow(gtk.Window):
         self.runningnextstore = None
         self.scrolledrunningnext = None
         self.runningnextview = None
+        self.__single_group = None
         
         self.channellists = {}
         self.manager = model
@@ -89,10 +72,7 @@ class ControlCenterWindow(gtk.Window):
         
         self.vbox_left = gtk.VBox(spacing=6)
         self.hpaned.pack1(self.vbox_left)
-        
-        groups_label = gtk.Label(_("Device groups:"))
-        self.vbox_left.pack_start(groups_label, False)
-        
+
         self.devgroupslist = gtk.ListStore(str, int, gobject.TYPE_PYOBJECT)
         self.devgroupslist.connect("row-inserted", self._on_devgroupslist_inserted)
         
@@ -102,7 +82,6 @@ class ControlCenterWindow(gtk.Window):
         cell_adapter = gtk.CellRendererText()
         self.devgroupscombo.pack_start(cell_adapter)
         self.devgroupscombo.add_attribute(cell_adapter, "markup", 0)
-        
         self.vbox_left.pack_start(self.devgroupscombo, False)
         
         self.channelsstore = None
@@ -121,19 +100,14 @@ class ControlCenterWindow(gtk.Window):
                 
         self.help_eventbox = HelpBox()
         self.choose_group_text = _("Choose a device group and channel on the left to view the program guide")
-        self.create_group_text = _("No devices are configured. Please go to preferences and to configure them.")
+        self.create_group_text = _("No devices are configured. Please go to preferences to configure them.")
         self.no_events_text = _("There is currently no schedule available for this channel")
         self.hpaned.pack2(self.help_eventbox)
         
-        self.scheduleview = ScheduleView()
+        self.schedulepaned = SchedulePaned()
+        self.schedulepaned.show()
+        self.scheduleview = self.schedulepaned.get_treeview()
         self.scheduleview.connect("button-press-event", self._on_event_selected)
-        self.scheduleview.show()
-        
-        self.scrolledschedule = gtk.ScrolledWindow()
-        self.scrolledschedule.add(self.scheduleview)
-        self.scrolledschedule.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        self.scrolledschedule.set_shadow_type(gtk.SHADOW_IN)
-        self.scrolledschedule.show()
         
         self.get_device_groups()
         if len(self.devgroupslist) == 0:
@@ -151,6 +125,8 @@ class ControlCenterWindow(gtk.Window):
             <menuitem action="Quit"/>
           </menu>
           <menu action="Edit">
+            <menuitem action="EditChannelLists"/>
+            <separator/>
             <menuitem action="Preferences"/>
           </menu>
           <menu action="View">
@@ -186,9 +162,9 @@ class ControlCenterWindow(gtk.Window):
         
         actiongroup = gtk.ActionGroup('Timers')
         actiongroup.add_actions([
-            ('EditTimers', None, _('_Manage'), None,
+            ('EditTimers', None, _('_Manage'), '<Control>T',
              _('Manage recording schedule'), self._on_button_display_timers_clicked),
-            ('Recordings', None, _('_Recordings'), None,
+            ('Recordings', None, _('_Recordings'), '<Control>W',
              _('Manage recordings'), self._on_button_recordings_clicked),
             ('Quit', gtk.STOCK_QUIT, _('_Quit'), None,
              _('Quit the Program'), gtk.main_quit)])
@@ -196,6 +172,8 @@ class ControlCenterWindow(gtk.Window):
         
         actiongroup = gtk.ActionGroup('Edit')
         actiongroup.add_actions([
+            ('EditChannelLists', None, _('_Channel Lists'), None,
+             _('Edit channel lists'), self._on_edit_channellists_clicked),
             ('Preferences', gtk.STOCK_PREFERENCES, _('_Preferences'), None,
              _('Display preferences'), self._on_button_prefs_clicked),
         ])
@@ -203,19 +181,19 @@ class ControlCenterWindow(gtk.Window):
         
         actiongroup = gtk.ActionGroup('View')
         actiongroup.add_actions([
-            ('WhatsOnNow', None, _("What's on now"), None,
+            ('WhatsOnNow', None, _("_What's on now"), None,
              _("See what's currently on and is coming next"), self._on_whats_on_now_clicked),
-            ('Refresh', gtk.STOCK_REFRESH, _('Refresh'), None,
+            ('Refresh', gtk.STOCK_REFRESH, _('_Refresh'), '<Control>R',
              _('Refresh program guide'), self._on_refresh_clicked),
-            ('PrevDay', None, _('Previous Day'), '<Control>B',
+            ('PrevDay', None, _('_Previous Day'), '<Alt>Left',
              _('Go to previous day'), self._on_button_prev_day_clicked),
-            ('NextDay', None, _('Next Day'), '<Control>N',
+            ('NextDay', None, _('_Next Day'), '<Alt>Right',
              _('Go to next day'), self._on_button_next_day_clicked),
         ])
         actiongroup.add_toggle_actions([
-            ('Channels', None, _('Channels'), None,
+            ('Channels', None, _('_Channels'), None,
              _('View/Hide channels'), self._on_view_channels_clicked),
-            ('Toolbar', None, _('Toolbar'), None,
+            ('Toolbar', None, _('_Toolbar'), None,
              _('View/Hide toolbar'), self._on_view_toolbar_clicked),
         ])
         action = actiongroup.get_action('Toolbar')
@@ -224,7 +202,7 @@ class ControlCenterWindow(gtk.Window):
         action.set_active(True)
         uimanager.insert_action_group(actiongroup, 3)
         
-        actiongroup = gtk.ActionGroup('Edit')
+        actiongroup = gtk.ActionGroup('Help')
         actiongroup.add_actions([
             ('About', gtk.STOCK_ABOUT, _('_About'), None,
              _('Display informations about the program'),
@@ -234,19 +212,15 @@ class ControlCenterWindow(gtk.Window):
 
         # Add a UI description
         uimanager.add_ui_from_string(ui)
-        
-        icon_theme = gtk.icon_theme_get_default()
-        
-        pixbuf = icon_theme.load_icon("stock_timer", gtk.ICON_SIZE_MENU, gtk.ICON_LOOKUP_USE_BUILTIN)
-        timers_image = gtk.image_new_from_pixbuf(pixbuf)
+
+        timers_image = gtk.image_new_from_icon_name("stock_timer", gtk.ICON_SIZE_MENU)
         timers_image.show()
         
         self.timersitem = uimanager.get_widget('/MenuBar/Timers/EditTimers')
         self.timersitem.set_image(timers_image)
         self.timersitem.set_sensitive(False)
         
-        pixbuf = icon_theme.load_icon("video", gtk.ICON_SIZE_MENU, gtk.ICON_LOOKUP_USE_BUILTIN)
-        recordings_image = gtk.image_new_from_pixbuf(pixbuf)
+        recordings_image = gtk.image_new_from_icon_name("video", gtk.ICON_SIZE_MENU)
         recordings_image.show()
         
         recordings = uimanager.get_widget('/MenuBar/Timers/Recordings')
@@ -282,11 +256,8 @@ class ControlCenterWindow(gtk.Window):
         self.toolbar = gtk.Toolbar()
         self.toolbar.show()
         self.vbox_outer.pack_start(self.toolbar, False)
-        
-        icon_theme = gtk.icon_theme_get_default()
-        
-        pixbuf = icon_theme.load_icon("stock_timer", gtk.ICON_SIZE_LARGE_TOOLBAR, gtk.ICON_LOOKUP_USE_BUILTIN)
-        timers_image = gtk.image_new_from_pixbuf(pixbuf)
+
+        timers_image = gtk.image_new_from_icon_name("stock_timer", gtk.ICON_SIZE_MENU)
         timers_image.show()
         
         self.button_display_timers = gtk.ToolButton(icon_widget=timers_image, label=_("Recording schedule"))
@@ -297,8 +268,7 @@ class ControlCenterWindow(gtk.Window):
         self.button_display_timers.show()
         self.toolbar.insert(self.button_display_timers, 0)
         
-        pixbuf = icon_theme.load_icon("video", gtk.ICON_SIZE_MENU, gtk.ICON_LOOKUP_USE_BUILTIN)
-        recordings_image = gtk.image_new_from_pixbuf(pixbuf)
+        recordings_image = gtk.image_new_from_icon_name("video", gtk.ICON_SIZE_MENU)
         recordings_image.show()
         
         button_recordings = gtk.ToolButton(icon_widget=recordings_image, label=_("Recordings"))
@@ -348,7 +318,18 @@ class ControlCenterWindow(gtk.Window):
         def append_groups(groups):
             for group in groups:
                 self._append_group(group)
+            self.check_single_group_mode()
         self.manager.get_registered_device_groups(reply_handler=append_groups)
+        
+    def check_single_group_mode(self):
+        val = len(self.devgroupslist) == 1
+        if val:
+            aiter = self.devgroupslist.get_iter_first()
+            self.__single_group = self.devgroupslist[aiter][2]
+            self.devgroupscombo.hide()
+        else:
+            self.__single_group = None
+            self.devgroupscombo.show()
     
     def _select_first_group(self):
         self.devgroupscombo.set_active(0)
@@ -389,12 +370,18 @@ class ControlCenterWindow(gtk.Window):
 
     def _on_manager_group_added(self, manager, group_id):
         group = self.manager.get_device_group(group_id)
-        self._append_group(group)
+        if group:
+            self._append_group(group)
+            self.check_single_group_mode()
         
     def _on_manager_group_removed(self, manager, group_id):
         self._remove_group(group_id)
+        self.check_single_group_mode()
+        self._select_first_group()
             
     def _get_selected_group(self):
+        if self.__single_group != None:
+            return self.__single_group
         aiter = self.devgroupscombo.get_active_iter()
         if aiter == None:
             return None
@@ -438,9 +425,9 @@ class ControlCenterWindow(gtk.Window):
                 
             self.scheduleview.set_model(self.schedulestore)
             # Display schedule if it isn't already displayed
-            if child != self.scrolledschedule:
+            if child != self.schedulepaned:
                 self.hpaned.remove(child)
-                self.hpaned.pack2(self.scrolledschedule)
+                self.hpaned.pack2(self.schedulepaned)
                 self._set_previous_day_sensitive(True)
                 self._set_next_day_sensitive(True)
                 self._set_refresh_sensitive(True)
@@ -507,18 +494,20 @@ class ControlCenterWindow(gtk.Window):
                 dialog = gtk.MessageDialog(parent=self,
                     flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                     type=gtk.MESSAGE_QUESTION, buttons=gtk.BUTTONS_YES_NO)
-                dialog.set_markup (_("<big><span weight=\"bold\">Schedule recording for the selected event?</span></big>"))
+                dialog.set_markup (
+                    "<big><span weight=\"bold\">%s</span></big>" % _("Schedule recording for the selected event?"))
                 rec_id = -1
+                success = True
                 if dialog.run() == gtk.RESPONSE_YES:
                     event_id = model[aiter][model.COL_EVENT_ID]
                     group = self._get_selected_group()
                     channel_sid = self._get_selected_channel_sid()
                     recorder = group.get_recorder()
-                    rec_id = recorder.add_timer_for_epg_event(event_id, channel_sid)
+                    rec_id, success = recorder.add_timer_for_epg_event(event_id, channel_sid)
                 dialog.destroy()
                 
-                if rec_id == 0:
-                    dialog = NoTimerCreatedDialog(self)
+                if not success:
+                    dialog = TimerFailureDialog(self)
                     dialog.run()
                     dialog.destroy()
         
@@ -569,8 +558,7 @@ class ControlCenterWindow(gtk.Window):
     
     def _on_button_prefs_clicked(self, button):
         prefs = Preferences(self.manager, self)
-        prefs.run()
-        prefs.destroy()
+        prefs.show()
         
     def _on_button_recordings_clicked(self, button):
         dialog = RecordingsDialog(self)
@@ -604,4 +592,8 @@ class ControlCenterWindow(gtk.Window):
         about.run()
         about.destroy()
         
-    
+    def _on_edit_channellists_clicked(self, action):
+        dialog = ChannelListEditorDialog(self.manager, parent=self)
+        dialog.run()
+        dialog.destroy()
+
