@@ -24,6 +24,11 @@ class BasePage(gtk.VBox):
         gtk.VBox.__init__(self, False, 5)
         self.set_border_width(5)
         
+        self._label = gtk.Label()
+        self._label.set_line_wrap(True)
+        self.pack_start(self._label, False, padding=3)
+        self._label.show()
+        
     def get_page_title(self):
         raise NotImplementedError
         
