@@ -20,12 +20,42 @@ import os
 import os.path
 import gtk
 import gobject
+import gettext
+import locale
 from gettext import gettext as _
 from gnomedvb.ui.wizard.pages.BasePage import BasePage
 
 DVB_APPS_DIRS = ("/usr/share/dvb",
                  "/usr/share/dvb-apps",
+                 "/usr/share/dvb-apps/scan",
                  "/usr/share/doc/dvb-utils/examples/scan")
+                 
+COUNTRIES = {
+    "at": "Austria",
+    "au": "Australia",
+    "be": "Belgium",
+    "ch": "Switzerland",
+    "cz": "Czech Republic",
+    "de": "Germany",
+    "dk": "Denmark",
+    "es": "Spain",
+    "fi": "Finland",
+    "fr": "France",
+    "gr": "Greece",
+    "hk": "Hong Kong",
+    "hr": "Hungary",
+    "is": "Iceland",
+    "it": "Italy",
+    "lu": "Luxemburg",
+    "nl": "Netherlands",
+    "no": "Norway",
+    "nz": "New Zealand",
+    "pl": "Poland",
+    "se": "Sweden",
+    "sk": "Slovakia",
+    "tw": "Taiwan",
+    "uk": "United Kingdom",
+}
 
 class InitialTuningDataPage(BasePage):
     
@@ -46,9 +76,14 @@ class InitialTuningDataPage(BasePage):
         
     def set_adapter_info(self, info):
         self.__adapter_info = info
-        
-        for widget in self.get_children():
+        # skip label
+        for widget in self.get_children()[1:]:
             widget.destroy()
+
+        if not self.is_dvb_apps_installed():
+            self.__page_title = _("Missing requirements")
+            self.setup_dvb_apps_missing()
+            return
             
         if info["type"] == "DVB-T":
             self.setup_dvb_t()
@@ -60,6 +95,7 @@ class InitialTuningDataPage(BasePage):
             self.setup_dvb_c()
             self.__page_title = _("Country and provider selection")
         else:
+            self.__page_title = _("Unsupported adapter")
             self.setup_unknown(info["type"])
             
     def get_tuning_data(self):
@@ -73,41 +109,45 @@ class InitialTuningDataPage(BasePage):
         self.table.set_col_spacings(3)
         self.table.show()
         self.pack_start(self.table)
+
+    def is_dvb_apps_installed(self):
+        val = False
+        for d in DVB_APPS_DIRS:
+            if os.path.exists(d):
+                val = True
+                break
+        return val
     
     def setup_unknown(self, devtype):
-        label = gtk.Label()
-        label.set_line_wrap(True)
         # translators: first %s is the DVB type, e.g. DVB-S
         text = _("Sorry, but '%s' cards aren't supported.") % devtype
-        label.set_markup(text)
-        label.show()
-        self.pack_start(label)
+        self._label.set_markup(text)
+
+    def setup_dvb_apps_missing(self):
+        text = "<big><b>%s</b></big>\n%s" % (_("Could not find initial tuning data."),
+            _("Please make sure that the dvb-apps package is installed."))
+        self._label.set_markup(text)
         
     def setup_dvb_t(self):
-        label = gtk.Label()
-        label.set_line_wrap(True)
         text = "%s %s %s" %(_('Please choose a country and the antenna that is closest to your location.'),
             _("If you don't know which antenna to choose select \"Don't know\" from the list of providers."),
             _("However, searching for channels will take considerably longer this way."))
-        label.set_markup(text)
-        label.show()
-        self.pack_start(label)
+        self._label.set_markup(text)
     
         self.providers_combo = None
         
-        countries = {self.NOT_LISTED: _("Not listed"),
-             "at": _("Austria"), "au": _("Australia"), "be": _("Belgium"),
-             "ch": _("Switzerland"), "cz": _("Czech Republic"), "de": _("Germany"),
-             "dk": _("Denmark"), "es": _("Spain"), "fi": _("Finland"), "fr": _("France"),
-             "gr": _("Greece"), "hr": _("Hungary"), "is": _("Iceland"), "it": _("Italy"),
-             "lu": _("Luxemburg"), "nl": _("Netherlands"), "nz": _("New Zealand"),
-             "pl": _("Poland"), "se": _("Sweden"), "sk": _("Slovakia"), "tw": _("Taiwan"),
-             "uk": _("United Kingdom"), "hk": _("Hong Kong") }
-    
+        countries = {self.NOT_LISTED: _("Not listed")}
+        country_codes = ("at", "au", "be", "ch", "cz", "de", "dk", "es", "fi", "fr",
+            "gr", "hr", "hk", "is", "it", "lu", "nl", "nz", "pl", "se", "sk",
+            "tw", "uk",)
+        t = gettext.translation("iso_3166", fallback=True)
+        for lang in country_codes:
+            countries[lang] = t.ugettext(COUNTRIES[lang])
+        
         self._create_table()
         
         country = gtk.Label()
-        country.set_markup(_("<b>Country:</b>"))
+        country.set_markup("<b>%s</b>" % _("Country:"))
         country.show()
         self.table.attach(country, 0, 1, 0, 1, yoptions=0)
 
@@ -129,7 +169,7 @@ class InitialTuningDataPage(BasePage):
         self.country_combo.set_active(0)
         
         self.providers_label = gtk.Label()
-        self.providers_label.set_markup(_("<b>Antenna:</b>"))
+        self.providers_label.set_markup("<b>%s</b>" % _("Antenna:"))
         self.providers_label.show()
         self.table.attach(self.providers_label, 0, 1, 1, 2, yoptions=0)
         
@@ -152,7 +192,7 @@ class InitialTuningDataPage(BasePage):
         self.pack_start(hbox, False)
         
         satellite = gtk.Label()
-        satellite.set_markup(_("<b>Satellite:</b>"))
+        satellite.set_markup("<b>%s</b>" % _("Satellite:"))
         satellite.show()
         hbox.pack_start(satellite, False, False, 0)
         
@@ -170,21 +210,22 @@ class InitialTuningDataPage(BasePage):
         self.read_satellites()
         
     def setup_dvb_c(self):
-        countries = {"at": _("Austria"), "be": _("Belgium"),
-             "ch": _("Switzerland"), "de": _("Germany"), "fi": _("Finland"),
-             "lu": _("Luxemburg"), "nl": _("Netherlands"), "se": _("Sweden"),
-             "no": _("Norway")
-             }
+        countries = {}
+        country_codes = ("at", "be", "ch", "de", "fi", "lu", "nl", "se", "no",)
+        t = gettext.translation("iso_3166", fallback=True)
+        for lang in country_codes:
+            countries[lang] = t.ugettext(COUNTRIES[lang])
             
         self._create_table()
             
         country = gtk.Label()
-        country.set_markup(_("<b>Country:</b>"))
+        country.set_markup("<b>%s</b>" % _("Country:"))
         country.show()
         self.table.attach(country, 0, 1, 0, 1, yoptions=0)
     
         self.countries = gtk.ListStore(str, str)
         self.countries.set_sort_column_id(0, gtk.SORT_ASCENDING)
+        self.countries.set_sort_func(0, self.combobox_sort_func)
         
         for code, name in countries.items():
             self.countries.append([name, code])
@@ -198,7 +239,7 @@ class InitialTuningDataPage(BasePage):
         self.table.attach(self.country_combo, 1, 2, 0, 1, yoptions=0)
         
         self.providers_label = gtk.Label()
-        self.providers_label.set_markup(_("<b>Providers:</b>"))
+        self.providers_label.set_markup("<b>%s</b>" % _("Providers:"))
         self.providers_label.show()
         self.table.attach(self.providers_label, 0, 1, 1, 2, yoptions=0)
         
@@ -306,5 +347,5 @@ class InitialTuningDataPage(BasePage):
         elif code2 == self.NOT_LISTED:
             return 1
         else:
-            return cmp(name1, name2)
+            return locale.strcoll(name1, name2)
     
